@@ -1,5 +1,4 @@
 from typing import (
-    Dict,
     Any,
     Callable,
     Tuple,
@@ -9,49 +8,42 @@ from typing import (
     Optional
 )
 
-from pandas import (
-    DataFrame,
-    Series,
-    to_datetime
-)
+from pandas import Series
 from pandas.core.generic import NDFrame
 
-import numpy as np
+from numpy import (
+    ndarray,
+    nan
+)
+
+from stock_pandas.meta.utils import ensure_return_type
 
 from .directive import (
     parse,
     Directive,
-    DirectiveCache
+    DirectiveCache,
+    directive_cache
 )
 
 from .common import rolling_calc
 
 from .meta import (
     ColumnInfo,
-
-    init_stock_metas,
-    copy_stock_metas,
-    copy_clean_stock_metas,
-    ensure_return_type
+    MetaDataFrame
 )
 
 
-class StockDataFrame(DataFrame):
+class StockDataFrame(MetaDataFrame):
     """The wrapper class for `pandas.DataFrame`
 
     Args definitions are the same as `pandas.DataFrame`
     """
 
     _stock_create_column: bool = False
-    _stock_indexer_slice: Optional[slice] = None
-    _stock_indexer_axis: int = 0
 
     # Directive cache can be shared between instances,
     # so declare as static property
-    _stock_directives_cache = DirectiveCache()
-
-    _stock_aliases_map: Dict[str, str]
-    _stock_columns_info_map: Dict[str, ColumnInfo]
+    _stock_directives_cache: DirectiveCache = directive_cache
 
     # Methods that used by pandas and sub classes
     # --------------------------------------------------------------------
@@ -65,73 +57,8 @@ class StockDataFrame(DataFrame):
 
         return StockDataFrame
 
-    def __finalize__(self, other, *args, **kwargs) -> 'StockDataFrame':
-        """
-        Propagate metadata from other to self.
-
-        This method overrides `DataFrame.__finalize__`
-        which ensures the meta info of StockDataFrame
-        """
-
-        super().__finalize__(other, *args, **kwargs)
-
-        if isinstance(other, StockDataFrame):
-            copy_clean_stock_metas(
-                other,
-                self,
-                other._stock_indexer_slice,
-                other._stock_indexer_axis
-            )
-
-        return self
-
-    def _slice(self, slice_obj: slice, axis: int = 0) -> 'StockDataFrame':
-        """
-        This method is called in several cases, self.iloc[slice] for example
-
-        We mark the slice and axis here to prevent extra calculations
-        """
-
-        self._stock_indexer_slice = slice_obj
-        self._stock_indexer_axis = axis
-
-        try:
-            result = super()._slice(slice_obj, axis)
-        except Exception as e:
-            raise e
-        finally:
-            self._stock_indexer_slice = None
-            self._stock_indexer_axis = 0
-
-        return result
-
     # --------------------------------------------------------------------
 
-    def __init__(
-        self,
-        data=None,
-        date_column=None,
-        *args,
-        **kwargs
-    ) -> None:
-        DataFrame.__init__(self, data, *args, **kwargs)
-
-        if self.columns.nlevels > 1:
-            # For now, I admit,
-            # there are a lot of works to support MultiIndex dataframes
-            raise ValueError(
-                'stock-pandas does not support dataframes with MultiIndex columns'  # noqa:E501
-            )
-
-        if isinstance(data, StockDataFrame):
-            copy_stock_metas(data, self)
-        else:
-            init_stock_metas(self)
-
-        if date_column:
-            self[date_column] = to_datetime(self[date_column])
-            self.set_index(date_column, inplace=True)
-
     def __getitem__(self, key) -> Union[Series, 'StockDataFrame']:
         if isinstance(key, str):
             key = self._map_single_key(key)
@@ -169,7 +96,7 @@ class StockDataFrame(DataFrame):
             name (str): The name of the column
 
         Returns:
-            np.Series
+            Series
         """
 
         origin_name = name
@@ -187,7 +114,7 @@ class StockDataFrame(DataFrame):
         self,
         directive_str: str,
         create_column: Optional[bool] = None
-    ) -> np.ndarray:
+    ) -> ndarray:
         """
         Executes the given directive and returns a numpy ndarray according to the directive.
 
@@ -198,7 +125,7 @@ class StockDataFrame(DataFrame):
             create_column (:obj:`bool`, optional): whether we should create a column for the calculated series.
 
         Returns:
-            np.ndarray
+            ndarray
         """
 
         if self._is_normal_column(directive_str):
@@ -274,21 +201,21 @@ class StockDataFrame(DataFrame):
         self,
         size: int,
         on: str,
-        apply: Callable[[np.ndarray], Any],
+        apply: Callable[[ndarray], Any],
         forward: bool = False,
-        fill=np.nan
-    ) -> np.ndarray:
+        fill=nan
+    ) -> ndarray:
         """Apply a 1-D function along the given column `on`
 
         Args:
             size (int): the size of the rolling window
             on (str | Directive): along which the function should be applied
             apply (Callable): the 1-D function to apply
-            forward (:obj:`bool`, optional): whether we should look backward (default value) to get each rolling window or not
+            forward (:obj:`bool`, optional): whether we should look forward to get each rolling window or not (default value)
             fill (:obj:`any`): the value used to fill where there are not enough items to form a rolling window
 
         Returns:
-            np.ndarray
+            ndarray
 
         Usage::
 
@@ -355,7 +282,7 @@ class StockDataFrame(DataFrame):
         self,
         directive: Directive,
         create_column: bool
-    ) -> Tuple[str, np.ndarray]:
+    ) -> Tuple[str, ndarray]:
         """Gets the series column corresponds the `directive` or
         calculate by using the `directive`
 
@@ -365,7 +292,7 @@ class StockDataFrame(DataFrame):
             calculated series
 
         Returns:
-            Tuple[str, np.ndarray]: the name of the series, and the series
+            Tuple[str, ndarray]: the name of the series, and the series
         """
 
         name = str(directive)
@@ -393,17 +320,17 @@ class StockDataFrame(DataFrame):
     def _set_new_item(
         self,
         name: str,
-        value: np.ndarray
+        value: ndarray
     ) -> None:
         """Set a new column and avoid SettingWithCopyWarning by using
         pandas internal APIs
 
         see: https://github.com/pandas-dev/pandas/blob/v1.1.0/pandas/core/frame.py#L3114
-        """  # noqa: E501
+        """
 
         NDFrame._set_item(self, name, value)
 
-    def _fulfill_series(self, column_name: str) -> np.ndarray:
+    def _fulfill_series(self, column_name: str) -> ndarray:
         column_info = self._stock_columns_info_map.get(column_name)
         size = len(self)
 
@@ -441,7 +368,7 @@ class StockDataFrame(DataFrame):
         return column_name in self.columns and \
             column_name not in self._stock_columns_info_map
 
-    def _calc(self, directive_str: str) -> np.ndarray:
+    def _calc(self, directive_str: str) -> ndarray:
         directive = self._parse_directive(directive_str)
 
         _, series = self._get_or_calc_series(
